package org.jvnet.hypercompass.runtime.tests;

import java.io.File;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;

import junit.framework.Assert;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.compass.core.Compass;
import org.compass.core.CompassHits;
import org.compass.core.CompassSession;
import org.compass.core.CompassTransaction;
import org.compass.core.Property;
import org.compass.core.Resource;
import org.compass.core.config.CompassConfiguration;
import org.compass.core.config.CompassEnvironment;
import org.jvnet.hypercompass.compass.core.converter.basic.EnumConverter;
import org.jvnet.jaxbcommons.tests.AbstractSamplesTest;

/**
 * Roundtrip test case.
 * 
 * @author Aleksei Valikov
 */
public abstract class RoundtripTest extends AbstractSamplesTest {

  /**
   * Test case logger.
   */
  protected Log logger = LogFactory.getLog(getClass());

  /**
   * Serializing transformer.
   */
  protected Transformer serializer;

  protected Compass compass;

  /**
   * Test setup.
   * 
   * @throws Exception
   *             In case of setup problems.
   */
  public void setUp() throws Exception {
    super.setUp();
    serializer = TransformerFactory.newInstance().newTransformer();
    serializer.setOutputProperty(OutputKeys.INDENT, "yes");

    //		final Properties properties = new Properties();
    //		properties.load(new FileInputStream(getPropertiesFile()));

    final CompassConfiguration configuration = new CompassConfiguration();
    

    configuration.setSetting(CompassEnvironment.CONNECTION, getIndexDirectory().getAbsolutePath());
    
//    configuration.addDirectory(getConfigurationDirectory());
    //		configuration.setProperties(properties);
    // configuration.getSessionEventListenerConfig().setMergeEventListener(
    // new IdTransferringMergeEventListener());

    //		final SchemaExport schemaExport = new SchemaExport(configuration,
    //				properties);
    //		schemaExport.setDelimiter(";");
    //		schemaExport.create(true, true);
    configuration.registerConverter(EnumConverter.class.getName(), new EnumConverter());

    compass = configuration.configure().buildCompass();
  }

  private File getIndexDirectory() {
    return new File(getBaseDir(), "target/compass");
  }

//  private File getConfigurationDirectory() {
//    return new File(getBaseDir(), "target/generated-sources/xjc");
//  }

  protected void checkSample(JAXBContext context, File sample) throws Exception {

    final Unmarshaller unmarshaller = context.createUnmarshaller();
    logger.debug("Unmarshalling.");
    // Unmarshall the document
    final Object object = unmarshaller.unmarshal(sample);
    logger.debug("Opening session.");
    {
      final CompassSession session = compass.openSession();
      final CompassTransaction tx = session.beginTransaction();
      session.save(object);
      tx.commit();
    }

    {
      final CompassSession session = compass.openSession();
      final CompassTransaction tx = session.beginTransaction();
      final CompassHits hits = session.find("test");
      Assert.assertTrue("Must return results.", hits.length() > 0);
      final Resource resource = hits.resource(0);
      logger.debug("Found resource with id [" + resource.get("id") + "]");
      final Property[] properties = resource.getProperties();
      Assert.assertTrue("At least one property must be present.", properties.length > 0);
      tx.commit();
    }

    {
      final CompassSession session = compass.openSession();
      final CompassTransaction tx = session.beginTransaction();
      session.delete(object);
      tx.commit();
    }

    //        
    //        
    //		// Open the session, save object into the database
    //		final Session saveSession = sessionFactory.openSession();
    //		logger.debug("Saving the object.");
    //		// final Object merged = saveSession.merge(object);
    //		// saveSession.replicate(object, ReplicationMode.OVERWRITE);
    //		// saveSession.get
    //		// final Serializable id =
    //		saveSession.saveOrUpdate(object);
    //
    //		final Serializable id = saveSession.getIdentifier(object);
    //		logger.debug("Flushing and closing the session.");
    //		saveSession.flush();
    //		// Close the session
    //		saveSession.close();
    //
    //		logger.debug("Opening session.");
    //		// Open the session, load the object
    //		final Session loadSession = sessionFactory.openSession();
    //		logger.debug("Loading the object.");
    //		final Object loadedObject = loadSession.load(object.getClass(), id);
    //		logger.debug("Closing the session.");
    //		loadSession.close();
    //
    //		final Marshaller marshaller = context.createMarshaller();
    //		logger.debug("Checking the document identity.");
    //
    //		logger.debug("Source object:\n" + ContextUtils.format(context, object));
    //		logger.debug("Result object:\n"
    //				+ ContextUtils.format(context, loadedObject));
    //
    //		Assert.assertEquals("Loaded and reloaded objects differ.", object,
    //				loadedObject);
  }
}